;******************************************************************************
;* PROCEDURE:     
;*      cassini_mimi_lemms_convert,input_data
;* 
;* DESCRIPTION:  
;*	This procedure converts the counts to count rates or eventually to flux   
;*
;* INPUTS:     
;*  units: 0 is counts, don't convert
;*        1 is counts/acc
;*        2 is flux (not implemented yet)  
;*	input_data: normal counter data to be converted
;*	input_data_prior: priority counter data to be converted
;*              
;* OUTPUTS: 
;*   	input_data after convertion
;*   	input_data_prior after convertion
;*
;* KEYWORDS: NONE
;* 
;* CALLING SEQUENCE:
;*       cassini_mimi_lemms_convert, units, input_data, input_data_prior
;*
;* MODIFICATION HISTORY: 
;*     written April 2003, M.B. Kusterer (JHU/APL)
;******************************************************************************
pro cassini_mimi_lemms_convert, units, input_data, input_data_prior

LOWBITRATE = 0
partial = 64.*16./(6.*1000000.)
nodat=4294967295ll 

; 
;units = counts/acc , divide by accumulation time
;
IF units EQ 1 THEN BEGIN    
print,'% CASSINI_MIMI_LEMMS_CONVERT: Converting to counts/acc'
    FOR n=0l,N_ELEMENTS(input_data)-1 DO BEGIN
    
        ;at low bit rate, get data every other subsector
        if input_data(n).telRate EQ LOWBITRATE THEN timeofonesub = input_data(n).spinPeriod*partial*2. $
        else timeofonesub = input_data(n).spinPeriod*partial
        
        at = WHERE(input_data(n).data(*) NE nodat)
        IF at(0) NE -1 THEN input_data(n).data(at) = input_data(n).data(at)/timeofonesub
    
    ENDFOR

    FOR n=0l,N_ELEMENTS(input_data_prior)-1 DO BEGIN
    
        ;at low bit rate, get data every other subsector and have 16 microsectors, else get 8
        if input_data_prior(n).telRate EQ LOWBITRATE THEN timeofonemicro = (input_data_prior(n).spinPeriod*partial*2.)/16.0 $
        else timeofonemicro = (input_data_prior(n).spinPeriod*partial)/8.0
        
        
        at = WHERE(input_data_prior(n).data(*) NE nodat)
        IF at(0) NE -1 THEN input_data_prior(n).data(at) = input_data_prior(n).data(at)/timeofonemicro

    ENDFOR
ENDIF

RETURN
END
