
;******************************************************************************
;* PROCEDURE:     
;*      cassini_mimi_lemms_priority, 
;* 
;* DESCRIPTION:  
;*	This procedure returns the channels Lemms currently has as priority channels and the 
;*  valid number of priority channels.    
;*
;* INPUTS:       
;*	mjdt        common time format 
;*              
;* OUTPUTS: 
;*  currprior 	string array of priority channel names
;*  numprior    number of priority channel names
;*  currindex   index into array of different priority counter sets, 0, 1, 2
;
;* KEYWORDS:
;* 
;* CALLING SEQUENCE:
;*       cassini_mimi_lemms_priority, mjdt, currprior, numprior 
;*
;* MODIFICATION HISTORY:       
;*   April 2003, M.B. Kusterer (JHU/APL) 
;******************************************************************************
PRO cassini_mimi_lemms_priority, mjdt_start, currprior, numprior, currindex
       ;
        currprior_all = [['Priority A0','Priority A1','Priority E0','Priority E1','Not Defined','Not Defined'],$
        ['Priority A0','Priority A1','Priority C0','Priority C1','Not Defined','Not Defined'],$
        ['Priority A1','Priority A5','Priority C1','Priority C5','Priority E4','Priority P2']]
        
        ;figure out priority channel name based on time
        ;
       flsw_1={MJDT, MJD:0l, T:0.0}
       flsw_1=convert_date_to_t90(doy=229, year=1999,$
                               hour=11,minutes=28,seconds=0, /MJDT)
       flsw_1_all = float(flsw_1.mjd)+(flsw_1.t/(24.0*60.0*60.0))
        
       flsw_2={MJDT, MJD:0l, T:0.0}
       flsw_2=convert_date_to_t90(doy=255, year=2003,$
                               hour=2,minutes=39,seconds=0, /MJDT)
       flsw_2_all = float(flsw_2.mjd)+(flsw_2.t/(24.0*60.0*60.0))
        
       curtime_all = float(mjdt_start.mjd)+(mjdt_start.t/(24.0*60.0*60.0))
       IF curtime_all GE flsw_2_all THEN BEGIN
            currindex = 2
       ENDIF ELSE IF curtime_all LT flsw_1_all THEN BEGIN
            currindex = 0
       ENDIF ELSE BEGIN
            currindex = 1 
       ENDELSE 
       
       numprior = 4
       If currindex EQ 2 THEN numprior = 6
       currprior = currprior_all[*,currindex]
       
       
       RETURN
   END 
