;****************************************************************************
;*  NAME:        define_cassini_mimi_lemms_channels
;*
;*  DESCRIPTION: Routine to define cassini_mimi_lemms information
;*
;*  INPUT:       none (uses common block time from PAPCO)
;*
;*  OUTPUT:      returns structure of channel info
;
;*  Notes:      Lemms priority channels 
;               before 1999-229T11:28 channels A0 A1 E0 E1
;               after 1999-229T11:28 channels A0 A1 C0 C1
;               after 2003-255 channels A1 A5 C1 C5 E4 P2
;*
;* MODIFICATION HISTORY:
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;   Mar 1 2004, M.B.Kusterer  Changed to work with the changed and 2 additional priority channels. Now is 63 big
;****************************************************************************
function define_cassini_mimi_lemms_channels
  
  chn={name:'',e_lo:0.,e_hi:0.,gf:0.,pos:0,species:'',detector:'', $
       sensor_dir:0.,opening_angle:15.}
;sensor_dir: look direction in  (0 for low E end, 180 for high-E end)
;opening_angle: opening angel of detector (15 for low-E)
  
  
  gf=[ 8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2,   $
       8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2, 8.8e-2]
  
  chmimi=replicate(chn,63)
  chmimi(0:62).detector='LEMMS'
  
  
;define A channels   
  chmimi(0:8).name=['A'+strtrim(string(indgen(9)),2)]
  chmimi(0:8).species='ions'
  chmimi(0:8).pos=indgen(9)
  chmimi(0:8).gf=[2.5e-3,2.5e-3,3.0e-3,3.0e-3,3.0e-3,3.0e-3,3.0e-3, $
                   3.0e-3,3.0e-3]
  chmimi(0:8).e_lo=[30., 36, 53, 105, 235, 493, 783, 1672, 0.]
  chmimi(0:8).e_hi=[36., 53,105, 235, 493, 783,1672, 4000, 0.]
;define B channels   
  chmimi(9:12).name=['B'+strtrim(string(indgen(4)),2)]
  chmimi(9:12).species='?'
  chmimi(9:12).pos=indgen(4)+9
  chmimi(9:12).gf=[3.0e-3,3.0e-3,3.0e-3,3.0e-3]
  chmimi(9:12).e_lo=[   0.,   0.,   0.,   0.]
  chmimi(9:12).e_hi=[   0.,   0.,   0.,   0.]
;BE-channel  
  chmimi(13).name='BE'
  chmimi(13).species='electrons'
  chmimi(13).pos=13
  chmimi(13).gf=3.0e-3
  chmimi(13).e_lo=1000
  chmimi(13).e_hi=2568
;define C channels   
  chmimi(14:21).name=['C'+strtrim(string(indgen(8)),2)]
  chmimi(14:21).species='electrons'
  chmimi(14:21).pos=indgen(8)+14
  chmimi(14:21).gf=[5.0e-3,2.0e-2,3.0e-2,3.5e-2,3.5e-2,2.0e-2,2.0e-2,2.0e-2]
  chmimi(14:21).e_lo=[15.,28,43,57, 99,220,305,550]
  chmimi(14:21).e_hi=[28.,43,57,99,220,305,550,884]
;define AS BS ES E2/F2S FS channels   
  chmimi(22:26).name=['AS', 'BS', 'ES', 'E2/F2S', 'FS']
  chmimi(22:26).species='singles'
  chmimi(22:26).pos=indgen(5)+22
  chmimi(22:26).gf=[ 3.0e-3, 3.0e-3, 2.5e-2, 1.0e-2, 1.0e-2]
  chmimi(22:26).e_lo=[   0.,   0.,   0.,   0.,   0.]
  chmimi(22:26).e_hi=[   0.,   0.,   0.,   0.,   0.]
;define E0 E1 E2 E3 E4 E5 E6 E7 channels   
  chmimi(27:34).name=['E'+strtrim(string(indgen(8)),2)]
  chmimi(27:34).species='electrons'
  chmimi(27:34).pos=indgen(8)+27
  chmimi(27:34).sensor_dir=180.
  chmimi(27:34).gf=[8.8e-3,8.8e-2,8.8e-2,8.8e-2,8.8e-2,8.8e-2,8.8e-2,8.8e-2]
  chmimi(27:34).e_lo=[ 50,250,570,650, 850, 884, 980, 3280]
;   chmimi(27:34).e_lo=[ 50,250,570,650, 850, 884,2500, 3280] ;tweak!!
  chmimi(27:34).e_hi=[260,850,750,950,2000,2500,3280, 5000]
                                ;original values:
;  chmimi(27:34).e_lo=[100,260,270,495, 950, 950, 980, 3280]
;  chmimi(27:34).e_hi=[260,950,495,744,2000,2500,3280, 5000]
;define P channels   
  chmimi(35).name='G1'
  chmimi(35).species='x-ray'
  chmimi(35).sensor_dir=180.
  chmimi(35).pos=35
  chmimi(35).gf=1.
  chmimi(35).e_lo=50
  chmimi(35).e_hi=400
;define P channels   
  chmimi(36:44).name=['P'+strtrim(string(indgen(9)+1),2)]
  chmimi(36:44).species='protons'
  chmimi(36:44).pos=indgen(9)+36
  chmimi(36:44).sensor_dir=180.
  chmimi(36:44).gf=[8.8e-2,8.8e-2,8.8e-2,8.8e-2,8.8e-2,8.8e-2,8.8e-2, $
                     8.8e-2,8.8e-2]
  chmimi(36:44).e_lo=[1630,2400,4500,5700, 8000,11000,13000,25000,60000.]
  chmimi(36:44).e_hi=[2400,4500,5700,8000,11000,13000,25000,60000.,100000.]
;remaining channels  H1 H2 H3 H4 H5 Z1 Z2 Z3 D1 D2 D3 D41    
  chmimi(45:56).name=['H1','H2','H3','H4','H5','Z1','Z2','Z3','D1','D2','D3','D41']
  chmimi(45:56).species='?'
  chmimi(45:56).pos=indgen(12)+45
  chmimi(45:56).sensor_dir=180.
  chmimi(45:56).gf=fltarr(12)
  chmimi(45:56).e_lo=fltarr(12)
  chmimi(45:56).e_hi=fltarr(12)
;Priority channels  A0 A1 C0 C1
  chmimi(57:62).name=['Prio A1','Prio A5','Prio C1','Prio C5','Prio E4','Prio P2']
  chmimi(57:62).species='?'
  chmimi(57:62).pos=indgen(6)+45
  chmimi(57:62).sensor_dir=180.
  chmimi(57:62).gf=fltarr(6)
  chmimi(57:62).e_lo=fltarr(6)
  chmimi(57:62).e_hi=fltarr(6)
    
  
  return,chmimi
end
