;******************************************************************************
;* file p_cassini_mimi_lemms_line2.pro  think does subsector averaging or something
;                                       like that MBK
;*
;* this file is a template for a plot routine which can be used by PAPCO.
;* it includes all the periphery needed, the user simply has to add in
;* the actual plot statement for his/her kind of data.
;*
;* This is a working example for a pre-defined example data array.
;******************************************************************************
;******************************************************************************
;* PROCEDURE:     
;*      p_cassini_mimi_lemms_line2, panel, type, $
;*           OVERPLOT=OVERPLOT,$
;*           PLOTS_ATTHISPOSITION=PLOTS_ATTHISPOSITION,$
;*           SUBTABLE=SUBTABLE, $
;*           _EXTRA=extra_par
;* 
;* DESCRIPTION:  
;*	This procedure plots data of type data_type for inclusion in PAPCO.  
;*      Data must have been read and placed into common blocks before this
;*      routine is called. Behavior is controlled by the vector panel
;*	which controls positioning of the plot (see papco_conventions.text
;*      in $PAPCO). In general, the following plot conventions are used:
;*              - if the panel is at the bottom plot time axis plus any
;*                additional axis (ie ephemeris) needed.
;*              - if panel is at the top also plot a title
;*              - a descriptor for panels is plotted to the right of each
;*                panel, rotated by 90 deg if there is a colorbar.
;*              - panels all have common time
;*              - y-scaling (or z-scaling for color plots) is either
;*              - automatic or manualy set using common yscale or zscale   
;*
;* INPUTS:       
;*      panel   three element integer vector. Controls positioning of the plot
;*              (see papco_conventions.text in $PAPCO)
;*      type    optional parameter for options. Normally is a four element
;*              integer vector. This is normally equivalent to the field
;*              typeVector of the PAPCO structure PAPCO_PLOTINFO which is set
;*              by the panel editor. Additional fields that can be used if
;*              needed are PAPCO_PLOTINFO.switch (three element vector) and 
;*              PAPCO_PLOTINFO.channel (integer). If your plot routine needs
;*              more input fields than these you have to extend the definition
;*              of the structure PAPCO_PLOTINFO
;*              (procedure PAPCO_getPlotInfoStruct in $PAPCO/papco.pro). Try to
;*              avoid this!
;*              
;* OUTPUTS: 
;*      none
;*
;* KEYWORDS:
;*      OVERPLOT  if you support this you can alter the appearance of your
;*              plot if it is an overplot to an existing panel. Standard is
;*              to plot data, no axis, using the y-scaling of the first plot
;*              and to shift the labeling on the left down one line. Useful
;*              to plot model data together with real data, ie mimi_lemms. field.
;*              see p_crres_mimi_lemms for an example.
;*      PLOTS_ATTHISPOSITION  used to control the left side labeling for more
;*              than one overplot at the same position.
;*      _EXTRA  the plot structure containing some keyword for calls to
;*              plot. Used to customize color and line style.   
;* 
;* CALLING SEQUENCE:
;*       p_cassini_mimi_lemms_line, panel, type, $
;*           OVERPLOT=OVERPLOT, PLOTS_ATTHISPOSITION=PLOTS_ATTHISPOSITION,$
;*           _EXTRA=extra_par
;*
;* MODIFICATION HISTORY:       
;*     written April 2001, S. BOURDARIE (ONERA/DESP)
;   April 2003, M.B. Kusterer (JHU/APL) Finished adding priority channels, reorganized into 
;                adding one more structure input_data_prior,to the data common block. It 
;                seemed the most memory efficient way due to the time difference for the priority.
;               Also added code to figure out based on time which priority channels are available.
;               I do store the min,max of A0,C0 in input_data now to speed up light removal call.
;               I had to make an if statement to separate the priority data from the normal.
;               Also changed all loop indexes to be longs in dealing with data arrays.
;               I also added the ability to do counts or count rates, flag stored in channel(16).
;   Mar 1 2004, M.B.Kusterer  Changed to work with the changed and 2 additional priority channels. 
;               Light correction only works for dates  1999-229 to 2003-255
;******************************************************************************
pro p_cassini_mimi_lemms_line2, panel,type, channel,OUTPUT=OUTPUT, $
                      OVERPLOT=OVERPLOT, $
                      PLOTS_ATTHISPOSITION=PLOTS_ATTHISPOSITION, $
                      SUBTABLE=SUBTABLE, $
                      _EXTRA=extra_par

; NOTE: I use a little trick in passing in the keywords to to this
; routine. The calling routine's call is simply:
;         p_cassini_mimi_lemms_line, panelVector, currPlotInfo.typeVector(0), $ 
;                           _EXTRA=extra_PlotInfo
; extra_PlotInfo contains the tags OVERPLOT, PLOTS_ATTHISPOSITION,
; SUBTABLE and then some tags which are meant to be passed to the plot
; call. By putting in the explicit keywords in the procedure definition they
; get split out of the structure extra_PlotInfo that was passed in, and only
; those tags which are meant for the plot routine remain in extra_par.

; Comments here should include an explanation of the plot options used
; in type, switch. Replace type in the call with whatever variables
; you need, or add more

; The following common blocks are needed:

   common mjdt, mjdt_start, mjdt_end    ;common time limit in mjdt
   common yscale, yscl			;man/auto yscaling
   common shift_label, down             ;common for x-axis label shifting
   common coordinateSystems, plotted_x, plotted_y  
                                        ;info on coords used by mouse
COMMON papco_color_names, black, red, green, yellow, blue, mimi_lemmsenta, cyan, $
  white, burgundy, olive, dark_green, teal, $
  royal_blue, violet, dark_grey, grey

; One of the common blocks should contain the plot data as returned by the
; data read procedure r_cassini_mimi_lemms. 

   common cassini_mimi_lemms_data, input_header, input_data, input_data_prior
   common time, xut1,xut2

   panelset,panel			;sets the panel position viewport

; make color array
  couleur_arr = [black, red, green, yellow, blue, mimi_lemmsenta, cyan, $
  white, burgundy, olive, dark_green, teal, $
  royal_blue, violet, dark_grey, grey]

; option to automatically calculate ytick numbers in order to avoid to
; many ticks if panels get too narrow. Maximum is set to 10. You may
; leave this out and allow plot to use the built-in defaults.

   ytickno=50*(!p.position(3)-!p.position(1))
   if ytickno gt 10 then ytickno=10

; since there is a common time axis plots look better if they all use the same
; number of xticks. In PAPCO, this number is 6 by default. This is set by PAPCO
; using the IDL graphics system variable !X.TICKS.
; You can override this for your own plot by setting the variable
; below. However, for plotting panels together, it makes sense to stick to the
; default. If you do set this variable, it needs to be included in the 
; extra_plotPar_common definition below.

;   xtickno=6

; NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! 
; Avoid setting the global IDL graphics system variables in your plot routine,
; as they may affect other modules' plot routines. The IDL graphics system 
; variables are set by PAPCO to default values that guarantee some uniformity
; of plot appearance and should not be tampered with - unless you don't
; care if your plot "looks good" when combined with the other modules.
; NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! NB!!! 

; the plot-calls normally use the arrays time for the time axis and yarr (ymat 
; for color plots) for the y or z axis.
; depending on your plot options, different data may be selected and different
; axis-labeling may be needed. This is normally done using a case statement.
; In this example the data array consist of flux values in ten channels 
; at different energies, typical of particle spectrometer data. Here I use the
; channel information passed in the plotinfo structure from the panel editor
; to select the channel to be drawn.

; set the begin end end times. PAPCo convention is to use running seconds
; starting with the no of seconds since the beginning of the start day.
   time=input_data.time
   xut1=mjdt_start.t                       
   xut2=long(mjdt_end.mjd-mjdt_start.mjd)*86400l+mjdt_end.t

;time between two points (needed not to plot missing data)

   index=where((time ge xut1) and (time le xut2),count)
   if count ne 0 then begin
      time=time(index)
      input_data_buf=input_data(index)
   endif

;Added same function for priority channels, time between two points (needed not to plot missing data), MBK
   if count ne 0 then begin
        time_prior=input_data_prior.time
      index_prior = index[0]*8 + L64INDGEN(count*8)
      time_prior=time_prior(index_prior)
      input_data_prior_buf=input_data_prior(index_prior)
   endif
   
   ;
   ; Convert into count rates or flux if selected
   ;
   if channel(16) NE 0 THEN BEGIN
        cassini_mimi_lemms_convert,channel(16),input_data_buf,input_data_prior_buf
   ENDIF
   
   res=where(channel(0:14) eq 1)
   ind = -1
   utitle1='CASSINI-MIMI/LEMMS!c'
   if type(0) eq 0 then begin
     utitle1=utitle1+'not av.'
     tmin = 100.
   endif
   if type(0) eq 1 then begin
     utitle1=utitle1+'spin av.'
     tmin=25.*60.
   endif
   ind_min=[0,9,14,22,27,35,36,45,53,60]

   typeNames=['A0','A1','A2','A3','A4','A5','A6','A7','A8', $
              'B0','B1','B2','B3','BE', $
              'C0','C1','C2','C3','C4','C5','C6','C7', $
	      'AS','BS','ES','E2SLF2S','FS', $
              'E0','E1','E2','E3','E4','E5','E6','E7', $
	      'G1',$
              'P1','P2','P3','P4','P5','P6','P7','P8','P9', $
              'H1','H2','H3','H4','H5','Z1','Z2','Z3', $
              'D1','D2','D3','D41','Blk','Blk','Blk',$
              'Prio_A0orA1','Prio_A1orA5','Prio_E0orC0orC1','Prio_E1orC1orC5','Prio_E4','Prio_P2']
   
   if type(2) gt type(3) then begin
     burp=type(2)
     type(2) = type(3)
     type(3) = burp
   endif
   if type(0) ge 2 then type(1)=0
  
   indice=where(input_data_buf.subsector ge type(2) and $
               input_data_buf.subsector le type(3),count)
   input_data_buf=input_data_buf(indice)
   time=time(indice)

  ;do the same for the priority channels, MBK
  indice_prior=where(input_data_prior_buf.subsector ge type(2) and $
               input_data_prior_buf.subsector le type(3),count_prior)
  input_data_prior_buf=input_data_prior_buf(indice_prior)
  time_prior=time_prior(indice_prior)

   nodat=4294967295ll

    ;figure out priority channel name based on time
    cassini_mimi_lemms_priority, mjdt_start, currprior, numprior, currindex 
    
; remove light contamination
   if channel(15) eq 0 or channel(15) eq 2 or channel(15) eq 9 then begin
   
       
       IF currindex EQ 2 THEN BEGIN
            print,'% CASSINI_MIMI_LEMMS_LIGHT_REMOVAL: No Light removal defined for this time period'
       ENDIF ELSE IF currindex EQ 0 THEN BEGIN
            print,'% CASSINI_MIMI_LEMMS_LIGHT_REMOVAL: No Light removal defined for this time period'
       ENDIF ELSE BEGIN
            
            cassini_mimi_lemms_light_removal,input_data, input_data_prior
       ENDELSE
      ENDIF

   nsub=type(3)-type(2)+1
   
   
;
; Process normal channels
;
    IF channel(15) NE 9 THEN BEGIN  

       yray = fltarr(count,n_elements(res)*nsub)
       tps = dblarr(count,nsub)
       utitle = strarr(n_elements(res)*nsub)
       for sub=type(2),type(3) do begin
         indix=where(input_data_buf.subsector eq sub,count)
         input_data_buf2=input_data_buf(indix)
         time2=time(indix)
         for i=0L,n_elements(res)-1 do begin
          ind = ind+1

          if type(0) eq 0 then begin  ;no average
             yray(0:n_elements(input_data_buf2)-1,ind)=input_data_buf2(*).data(ind_min(channel(15))+res(i))
	     tps(0:n_elements(input_data_buf2)-1,sub-type(2))=time2
	     nav=n_elements(time2)-1
          endif

          if type(0) eq 1 or type(0) eq 3 then begin  ;spin average and subs+spin av
	     if input_data_buf2(0).data(ind_min(channel(15))+res(i)) ne nodat then begin
                yray(0,ind)=input_data_buf2(0).data(ind_min(channel(15))+res(i))
                tps(0,sub-type(2))=time2(0)
       	        np=1
	     endif else begin
	        yray(0,ind)=0.
	        tps(0,sub-type(2))=0.d0
	        np=0
	     endelse
	     nav = 0
	     spin=input_data_buf2(0).spin
             for j=1L,n_elements(input_data_buf2)-1 do begin
	        if input_data_buf2(j).spin eq spin then begin
	          if input_data_buf2(j).data(ind_min(channel(15))+res(i)) ne nodat then begin
	             yray(nav,ind)=yray(nav,ind)+input_data_buf2(j).data(ind_min(channel(15))+res(i))
                     tps(nav,sub-type(2))=tps(nav,sub-type(2))+time2(j)
                     np=np+1
	          endif
                endif else begin
	          if np ne 0 then begin
                     yray(nav,ind)=yray(nav,ind)/float(np)
	             tps(nav,sub-type(2))=tps(nav,sub-type(2))/float(np)
	          endif else begin
	             yray(nav,ind)=nodat
		     tps(nav,sub-type(2))=0.d0
	          endelse
	          nav=nav+1
	          if input_data_buf2(j).data(ind_min(channel(15))+res(i)) ne nodat then begin
                     yray(nav,ind)=input_data_buf2(j).data(ind_min(channel(15))+res(i))
                     tps(nav,sub-type(2))=time2(j)
       	             np=1
	          endif else begin
	             yray(nav,ind)=0.
	             tps(nav,sub-type(2))=0.d0
	             np=0
	          endelse
	          spin=input_data_buf2(j).spin
                endelse
	     endfor
	     if np ne 0 then begin
                yray(nav,ind)=yray(nav,ind)/float(np)
	        tps(nav,sub-type(2))=tps(nav,sub-type(2))/float(np)
	     endif else begin
	        yray(nav,ind)=nodat
	        tps(nav,sub-type(2))=0.d0
	     endelse
          endif

          if type(0) eq 2 then begin  ;subsector average
	     if input_data_buf2(0).data(ind_min(channel(15))+res(i)) ne nodat then begin
                yray(0,ind)=input_data_buf2(0).data(ind_min(channel(15))+res(i))
                tps(0,sub-type(2))=time2(0)
       	        np=1
	     endif else begin
	        yray(0,ind)=0.
	        tps(0,sub-type(2))=0.d0
	        np=0
	     endelse
	     nav = 0
	     sect=input_data_buf2(0).sector
             for j=1L,n_elements(input_data_buf2)-1 do begin
	        if input_data_buf2(j).sector eq sect then begin
	          if input_data_buf2(j).data(ind_min(channel(15))+res(i)) ne nodat then begin
	             yray(nav,ind)=yray(nav,ind)+input_data_buf2(j).data(ind_min(channel(15))+res(i))
                     tps(nav,sub-type(2))=tps(nav,sub-type(2))+time2(j)
                     np=np+1
	          endif
                endif else begin
	          if np ne 0 then begin
                     yray(nav,ind)=yray(nav,ind)/float(np)
	             tps(nav,sub-type(2))=tps(nav,sub-type(2))/float(np)
	          endif else begin
	             yray(nav,ind)=nodat
		     tps(nav,sub-type(2))=0.d0
	          endelse
	          nav=nav+1
	          if input_data_buf2(j).data(ind_min(channel(15))+res(i)) ne nodat then begin
                     yray(nav,ind)=input_data_buf2(j).data(ind_min(channel(15))+res(i))
                     tps(nav,sub-type(2))=time2(j)
       	             np=1
	          endif else begin
	             yray(nav,ind)=0.
	             tps(nav,sub-type(2))=0.d0
	             np=0
	          endelse
	          sect=input_data_buf2(j).sector
                endelse
	     endfor
	     if np ne 0 then begin
                yray(nav,ind)=yray(nav,ind)/float(np)
	        tps(nav,sub-type(2))=tps(nav,sub-type(2))/float(np)
	     endif else begin
	        yray(nav,ind)=nodat
	        tps(nav,sub-type(2))=0.d0
	     endelse
          endif
          utitle(ind)=typeNames(ind_min(channel(15))+res(i))+' ss'+$
                      strtrim(string(sub),2)
        endfor
       endfor

   
;
; Priority Channels
;
    ENDIF ELSE BEGIN
    
        
       ;If user selected priority channels 4 or 5 for time when only had 0-3 then turn them off
       IF currindex LT 2 THEN BEGIN
            FOR n=0,n_elements(res)-1 DO BEGIN
                IF res[n] GT 3 THEN BEGIN
                    res[n] = 999
                ENDIF
            ENDFOR
            at = WHERE(res[*] NE 999)
            IF at[0] NE -1 THEN res = res[at]
            utitle = strarr(n_elements(res))
       ENDIF
       
       yray = fltarr(count_prior,n_elements(res)*nsub)
       tps = dblarr(count_prior,nsub)
       utitle = strarr(n_elements(res)*nsub)
       for sub=type(2),type(3) do begin
         indix=where(input_data_prior_buf.subsector eq sub,count_prior)
         input_data_prior_buf2=input_data_prior_buf(indix)
         time_prior2=time_prior(indix)
         for i=0L,n_elements(res)-1 do begin
          ind = ind+1

          if type(0) eq 0 then begin  ;no average
             yray(0:n_elements(input_data_prior_buf2)-1,ind)=input_data_prior_buf2(*).data(res(i))
	     tps(0:n_elements(input_data_prior_buf2)-1,sub-type(2))=time_prior2
	     nav=n_elements(time_prior2)-1
          endif

          if type(0) eq 1 or type(0) eq 3 then begin  ;spin average and subs+spin av
	     if input_data_prior_buf2(0).data(res(i)) ne nodat then begin
                yray(0,ind)=input_data_prior_buf2(0).data(res(i))
                tps(0,sub-type(2))=time_prior2(0)
       	        np=1
	     endif else begin
	        yray(0,ind)=0.
	        tps(0,sub-type(2))=0.d0
	        np=0
	     endelse
	     nav = 0
	     spin=input_data_prior_buf2(0).spin
             for j=1L,n_elements(input_data_prior_buf2)-1 do begin
	        if input_data_prior_buf2(j).spin eq spin then begin
	          if input_data_prior_buf2(j).data(res(i)) ne nodat then begin
	             yray(nav,ind)=yray(nav,ind)+input_data_prior_buf2(j).data(res(i))
                     tps(nav,sub-type(2))=tps(nav,sub-type(2))+time_prior2(j)
                     np=np+1
	          endif
                endif else begin
	          if np ne 0 then begin
                     yray(nav,ind)=yray(nav,ind)/float(np)
	             tps(nav,sub-type(2))=tps(nav,sub-type(2))/float(np)
	          endif else begin
	             yray(nav,ind)=nodat
		     tps(nav,sub-type(2))=0.d0
	          endelse
	          nav=nav+1
	          if input_data_prior_buf2(j).data(res(i)) ne nodat then begin
                     yray(nav,ind)=input_data_prior_buf2(j).data(res(i))
                     tps(nav,sub-type(2))=time_prior2(j)
       	             np=1
	          endif else begin
	             yray(nav,ind)=0.
	             tps(nav,sub-type(2))=0.d0
	             np=0
	          endelse
	          spin=input_data_prior_buf2(j).spin
                endelse
	     endfor
	     if np ne 0 then begin
                yray(nav,ind)=yray(nav,ind)/float(np)
	        tps(nav,sub-type(2))=tps(nav,sub-type(2))/float(np)
	     endif else begin
	        yray(nav,ind)=nodat
	        tps(nav,sub-type(2))=0.d0
	     endelse
          endif

          if type(0) eq 2 then begin  ;subsector average
	     if input_data_prior_buf2(0).data(res(i)) ne nodat then begin
                yray(0,ind)=input_data_prior_buf2(0).data(res(i))
                tps(0,sub-type(2))=time_prior2(0)
       	        np=1
	     endif else begin
	        yray(0,ind)=0.
	        tps(0,sub-type(2))=0.d0
	        np=0
	     endelse
	     nav = 0
	     sect=input_data_prior_buf2(0).sector
             for j=1L,n_elements(input_data_prior_buf2)-1 do begin
	        if input_data_prior_buf2(j).sector eq sect then begin
	          if input_data_prior_buf2(j).data(res(i)) ne nodat then begin
	             yray(nav,ind)=yray(nav,ind)+input_data_prior_buf2(j).data(res(i))
                     tps(nav,sub-type(2))=tps(nav,sub-type(2))+time_prior2(j)
                     np=np+1
	          endif
                endif else begin
	          if np ne 0 then begin
                     yray(nav,ind)=yray(nav,ind)/float(np)
	             tps(nav,sub-type(2))=tps(nav,sub-type(2))/float(np)
	          endif else begin
	             yray(nav,ind)=nodat
		     tps(nav,sub-type(2))=0.d0
	          endelse
	          nav=nav+1
	          if input_data_prior_buf2(j).data(res(i)) ne nodat then begin
                     yray(nav,ind)=input_data_prior_buf2(j).data(res(i))
                     tps(nav,sub-type(2))=time_prior2(j)
       	             np=1
	          endif else begin
	             yray(nav,ind)=0.
	             tps(nav,sub-type(2))=0.d0
	             np=0
	          endelse
	          sect=input_data_prior_buf2(j).sector
                endelse
	     endfor
	     if np ne 0 then begin
                yray(nav,ind)=yray(nav,ind)/float(np)
	        tps(nav,sub-type(2))=tps(nav,sub-type(2))/float(np)
	     endif else begin
	        yray(nav,ind)=nodat
	        tps(nav,sub-type(2))=0.d0
	     endelse
          endif
          utitle(ind)=currprior(res(i))+' ss'+$
                      strtrim(string(sub),2)
        endfor
       endfor


    
    ENDELSE   
   

   tps=tps(0:Nav,*)
   yray=yray(0:NaV,*)

   IF channel(16) EQ 0 THEN uytitle='[counts]' $
   ELSE uytitle='[counts/acc]'

   ind = -1
   for sub=type(2),type(3) do begin
    for i=0L,n_elements(res)-1 do begin
      ind = ind+1
      yvect = yray(*,ind)
      time  = tps(*,sub-type(2))
      index=where(yvect ne nodat, count)
      if count ne 0 then begin
          yvect=yvect(index)
          t=time(index)
      endif else t=time
      if yscl(panel(0),3) eq 0 then $
         ylim4,t,yvect,ymin,ymax $
      else ylim_io,t,yvect,ymin,ymax
      if (ind eq 0) then begin
         yst = ymin
         yen = ymax
      endif else begin
         if ymin lt yst then yst = ymin
         if ymax gt yen then yen = ymax
      endelse
     endfor
    endfor

; the variables used above have the following meaning:
; 	uytitle : the y-axis label
;	utitle  : the right-of plot label defining the data
;
; the procedure YLIM3 is optional and sets the ylimit automatically for the
; data leaving a 10% gap at the top and bottom limits.
; this routine is very simple and does not handle spikes or bad data, you may
; need to supply your own procedure for your own data. 
; (in $PAPCO/PAPCO_Lib/ylim.pro)

; yscl of common block yscale is an array of dimensions (*,4) and contains info
; on the yscaling to be used. The first dimension identifies the panel, and the
; next dimension has three entries:  
;         yscl(*,0)  is  0 for "use automatic scaling", 1 for "use manual"
;         yscl(*,1)  is then ymin (or zmin)
;         yscl(*,2)  is then ymax (or zmax)
;         yscl(*,3)  is 0 for "linear" and 1 for "log"  scale
; when scaling is set to automatic, the automatic y-limits are loaded in.
   
   if (yscl(panel(0),0) eq 1) then begin
      yst=yscl(panel(0),1)
      yen=yscl(panel(0),2)
   endif else begin
      yscl(panel(0),1)=yst
      yscl(panel(0),2)=yen
   endelse

; set up extra plot keywords, common ones first. 
; yrange, xrange  are always needed. 
; the rest you can set if you want to override PAPCO defaults.
; See $PAPCO_BASE/PAPCO/papco_draw for the definition of the PAPCO default
; graphic system variable settings.

   extra_plotPar_common={yrange:[yst,yen], xrange:[xut1,xut2], $
                         ytitle:uytitle, $
                         ylog:yscl(panel(0),3)}
   
; check if the passed-in structure extra_par was set. If not, create
; it and set color to default black; which for the color table used by
; PAPCO is 1. If extra_par was set, see if the tag color exists. If it
; does, leave it unchanged, if not, add it and set color to default
; black. 

  if n_elements(extra_par) EQ 0 then extra_par={color:couleur_arr(0)} else begin
    names=tag_names(extra_par)
    resu=where(names eq 'COLOR',count)
    if count eq 0 then extra_par=create_struct({color:couleur_arr(0)}, extra_par)
  endelse     
   
; add keyword structure set here with the one passed in

   extra_plotPar_common=create_struct(extra_plotPar_common, extra_par)

; extend this structure by those keywords specific to the plot being bottom
; panel or not. 
; You might have to add your own xtickformat for time axis labeling. If you 
; stick to the PAPCO convention of having your plot  time array having 
; consecutive seconds from midnight of the start day, then  you can use the 
; pre-defined xtickformat functions defined in ticks.pro in
; PAPCO_Lib/axis_label.
; You can at this point also define a custom ytickformat, or leave it out for
; IDL default plot behavior.
; down sets the axis labeling level

   down=0
   if panel(0) eq 0 then $
     extra_plotPar=create_struct({xtickformat:'MJDT_DATE_4'}, $
                                  extra_plotPar_common) $
   else $
     extra_plotPar=create_struct({xtickformat:'noticks'}, $
                                  extra_plotPar_common)

; To keep things general, plotting of the data is split up into plotting the 
; axis with no data, and then plotting the data with no axis. This is done to 
; enable the feature of overplots, which just plot data into an existing axis.
; plot the axis in color=1, and only plot axis if if you're NOT over-plotting!
; down sets the axis labeling level (adds carriage returns [!C] to labels).

   down=0
   color1_plotpar=extra_plotPar
   color1_plotpar.color=1

   if not keyword_set(overplot) then begin
          plot,time,yray, _extra=color1_plotPar, xstyle=1, ystyle=1, $
           /NODATA
    endif

;create new structure to use PLOTS with IDL 4.

    extra_gapplotPar = create_struct({PSYM:0},{COLOR:0L},{THICK:1},{LINESTYLE:0})
    extra_gapplotPar.PSYM = extra_plotPar.PSYM
    extra_gapplotPar.COLOR = extra_plotPar.COLOR
    extra_gapplotPar.THICK = extra_plotPar.THICK
    extra_gapplotPar.LINESTYLE = extra_plotPar.LINESTYLE

; now do the actual data plot, data only, no axis
    couleur_start = extra_plotPar.color
    couleur_index = where(couleur_start eq couleur_arr)
    if couleur_index(0) eq -1 then couleur_index(0) = 0
    couleur = extra_plotPar.color
    ind = -1
    for sub=type(2),type(3) do begin
     for i=0L,n_elements(res)-1 do begin
       ind = ind + 1
       yvect = yray(*,ind)
       time=tps(*,sub-type(2))
       index=where(yvect ne nodat, count)
       if count ne 0 then begin
          yvect=yvect(index)
          t=time(index)
       endif
       extra_gapplotPar.color = couleur
       cassini_mimi_lemms_gap_oplot,tmin,t,yvect, _extra=extra_gapplotPar
       couleur = couleur_arr(couleur_index(0)+ind+1<15)
     endfor
    endfor

; store the coordinate information into common block for slices

   plotted_x = !x
   plotted_y = !y

; plot extra x-axis labels (eg. ephemeris) and a x-axis label if required.
; plot in color=1. Example here adds an axis with date information.
; The routine x_side_label puts x-axis label to the right of the axis.
; If you stick to the PAPCO convention of always having your plot 
; time array having consecutive seconds from midnight of the start day, then 
; you can use the pre-defined xtickformat functions defined in ticks.pro in
; PAPCO_Lib/axis_label.
; Or use any other function you define yourself.
; down sets the axis labeling level

  if (panel(0) eq 0) and not keyword_set(OVERPLOT) then begin
     extra_plotPar=create_struct({xtickformat:'MJDT_DATE_3'}, $
                                 extra_plotPar_common) 
     extra_plotPar.color=1     
     down=1
     plot,time,yray, _extra=extra_plotPar, xstyle=1, ystyle=1, /NODATA
     x_side_label,panel,'time (UT)!CDate'
  endif

; plot info right of the panel. If you want overplots to have their own labels
; you need to add position-control for this label using the keyword
; PLOTS_ATTHISPOSITION

   extra_Par.color = couleur_start
   couleur = extra_Par.color
   if not keyword_set(overplot) then begin
     extra_Par.color = 1
     right_side_label,panel,utitle1,_extra=extra_Par
     utitle2 = '!C!C'
     ind = -1
     for sub=type(2),type(3) do begin
      for i=0L,n_elements(res)-1 do begin
        ind = ind+1
        extra_Par.color = couleur
        utitleS = utitle2 + '!C' + utitle(ind)
        right_side_label,panel,utitleS,_extra=extra_Par
        utitle2 = utitle2 + '!C'
        couleur = couleur_arr(couleur_index(0)+ind+1<15)
      endfor
     endfor
   endif else begin
     for i=0, PLOTS_ATTHISPOSITION-1 do utitle='!C'+utitle1
     right_side_label,panel,utitle,_extra=extra_Par
   endelse


; check if data needs to be output to file. Use the optional keywords
; to add more information to the data file
;      DESCRIPTION   a string or string array describing the data
;      TIME_LABEL    a string decribing the x-axis format
;      CHANNELS      an (ch,2) array giving the channel start/end
;      Y_LABEL       a string with the y-axis label
;      Z_LABEL       a string with the z-axis label
;      RS_LABEL      a string with the righ-side label of the plot
;      NODATA        contains the no data flag value.
  
  if KEYWORD_SET(OUTPUT) then begin
    print,'% p_cassini_mimi_lemms_line: Writing plot data out to file'
    print,'% NOT IMPLEMENTED !!'
  endif
   



end 


;******************************************************************************
;* PROCEDURE:     
;*      ylim_io,time,yray,yst,yen
;* 
;* DESCRIPTION:  
;*      This procedure search for min and max in a log scale in array yray
;*
;* INPUTS:  
;*      time     
;*      yray     array to be plotted
;*              
;* OUTPUTS: 
;*      yst,yen   min and max
;* 
;* CALLING SEQUENCE:
;*      ylim_io,time,yray,yst,yen
;*
;* MODIFICATION HISTORY:       
;*     written September 2000, Sebastien Bourdarie, at ONERA/DESP
;******************************************************************************
pro ylim_io,time,yray,yst,yen

common time, xut1,xut2

index=where((time ge xut1) and (time le xut2) and (yray gt 0.),count)

if count gt 2 then begin
  yen=max(yray(index),min=yst)
  yst = 10.^floor(ALOG10(yst))
  yen = 10.^ceil(ALOG10(yen))
endif else begin
  yen=10.
  yst=1.
endelse

end


;******************************************************************************
;* PROCEDURE:     
;*      cassini_mimi_lemms_gap_oplot, x, y, _EXTRA=e
;* 
;* DESCRIPTION:  
;*      This procedure plots data without connecting data point if they are
;*      further than 5 min. apart.
;*
;* INPUTS: 
;*      step    time step between two measurements      
;*      x,y     arrays to be plotted
;*              
;* OUTPUTS: 
;*      none
;* 
;* CALLING SEQUENCE:
;*      cassini_mimi_lemms_gap_oplot, x, y, _EXTRA=e
;*
;* MODIFICATION HISTORY:       
;*     written September 2000, Sebastien Bourdarie, at ONERA/DESP
;******************************************************************************
pro cassini_mimi_lemms_gap_oplot,step,x,y,_EXTRA=e

; work out jumps in y. If a jump is larger than step, don't plot

   ndat=n_elements(x)
   if (ndat eq 0) then return
   for i=0l,ndat-2l do begin
      int=abs(x(i+1)-x(i))
      if int lt step then begin
        plots,x(i),y(i) 
        plots,x(i+1),y(i+1),/continue,noclip=0 ,_EXTRA=e
      endif
    endfor

end


